/*!
    \file object_manager.h
  *  this manager will ceep track of the objects  in the world

    This is specially designed to:
    - Give an interface to the world data
    - separate the world data from the world build,update,draw process
    - recycle deleted objects: specific for Projectils since there is a lot of world entity creation/deletion (and this needs a lot of time)
    - control the garbage collector

    TO ADD SUPPORT FOR A CLASS do the following steps:
    1. include the hader file : #include "class_id.h"
    2. add the class to the type enum classID {}; in class_id.h
    3. define a function void mCache( ClassName ) in class ObjectManager

*/


#ifndef _OBJECT_MANAGER_H
#define _OBJECT_MANAGER_H

#include "base_object.h"
#include "class_id.h"

template<class T> class tList;
class GarbageCollector;

//! the object manager itself
class ObjectManager : public BaseObject {

 public:
  virtual ~ObjectManager();
  /** @returns a Pointer to the only object of this Class */
  inline static ObjectManager* getInstance() { if (!singletonRef) singletonRef = new ObjectManager();  return singletonRef; };

  void registerClass(ClassID classID);

  /** a class handled by the objectManage */
  void addToDeadList(int index, BaseObject* object);
  BaseObject* getFromDeadList(int index, int number = 1);

  BaseObject* resurect();
  void kill(BaseObject* object);


  void debug() const;

 private:
  ObjectManager();

 private:
  static ObjectManager*      singletonRef;          //!< The singleton reference to the only reference of this class

  tList<BaseObject>**        managedObjectList;     //!< A list of managed objects (handles different types and lists of them)
};



/**
 * Creates a factory to a Loadable FastObject.
 */
#define CREATE_FAST_OBJECT(CLASS_NAME, CLASS_ID) \
    tFastObject<CLASS_NAME>* global_##CLASS_NAME##_FastObject = new tFastObject<CLASS_NAME>(#CLASS_NAME, CLASS_ID)

//! The FastObject is a loadable object handler
class FastObject : public BaseObject {

  public:
    FastObject (const char* fastObjectName = NULL, ClassID classID = CL_NULL);
    virtual ~FastObject ();

    virtual BaseObject* fabricate(ClassID classID) = NULL;

    static void registerFastObject(FastObject* fastObject);
    /** sets the Next factory in the list @param nextFactory the next factory */
    inline void setNext( FastObject* nextFastObject) { this->next = nextFastObject; };
    /** @returns the first FastObject */
    static FastObject* getFirst() { return FastObject::first; };
    /** @returns the next FastObject */
    FastObject* getNext() const { return this->next; };

  protected:
    ClassID              storedClassID;        //!< The classID of the specified class.

  private:
    FastObject*          next;                 //!< pointer to the next FastObject.
    static FastObject*   first;                //!< A pointer to the first FastObject.
};

/**
 *  a FastObject that is able to load any kind of Object
 * (this is a Functor)
 */
template<class T> class tFastObject : public FastObject
{
  public:
    tFastObject(const char* fastObjectName, ClassID fastObject);

  private:
    virtual BaseObject* fabricate(ClassID fastObject);
};

/**
 * construnts a FastObject with
 * @param fastObjectName the name of the FastObject
 * @param fastObject the ID of the class
 */
template<class T>
tFastObject<T>::tFastObject(const char* fastObjectName, ClassID fastObject) : FastObject(fastObjectName, fastObject)
{
  PRINTF(5)("Class: %s loadable as a FastObject\n", this->getName());
}

template<class T>
BaseObject* tFastObject<T>::fabricate(ClassID fastObject)
{
//  if(this->classID == fastObject)
    //return new T ();
  //else if( getNext() != NULL)
//    return getNext()->fabricate();
//  else
    return NULL;
}

#endif /* _OBJECT_MANAGER_H */
